package lab.game;

import java.util.*;
import java.util.concurrent.*;
import java.io.IOException;

public class Board{
  private int n;
  private int m;
  private int c;
  private Thief thief;
  private int thiefMoves;
  private int policeMoves;
  private ArrayList<Police> polices;

  public Board(int n, int m, int c){
    this.n = n;
    this.m = m;
    this.c = c;
    this.thief        = null;
    this.polices      = new ArrayList<Police>();
    this.thiefMoves   = 0;
    this.policeMoves  = 0;
  }

  public void setThief(Thief t){
    this.thief = t;
  }

  public void addPolice(Police p){
    this.polices.add(p);
  }

  public void initBoard(){
    Random rand = new Random();
    // Create Thief
    boolean flag = true;
    int x = 0, y = 0;
    while(flag){
      x = rand.nextInt(this.n);
      y = rand.nextInt(this.m);
      flag = !isFree(x, y);
    }
    Thief newThief = new Thief(x, y);
    this.thief = newThief;
    // Create Polices
    for(int i = 0; i < this.c; i++){
      flag = true;
      x = 0;
      y = 0;
      while(flag){
        x = rand.nextInt(this.n);
        y = rand.nextInt(this.m);
        flag = !(isFree(x, y) && !isThiefInRange(x, y));
      }
      Police newPolice = new Police(x, y);
      this.polices.add(newPolice);
    }
  }

  public void startGame() throws IOException, InterruptedException{
    boolean flag = true;
    int i = 0;
    while(flag){
      new ProcessBuilder("cmd", "/c", "cls").inheritIO().start().waitFor();
      this.print(i);
      flag = this.timeMove();
      TimeUnit.SECONDS.sleep(2);
      i++;
    }
    new ProcessBuilder("cmd", "/c", "cls").inheritIO().start().waitFor();
    this.printFinal();
  }

  public Boolean isTakenByThief(int x, int y){
    if( this.thief != null && this.thief.getX() == x && this.thief.getY() == y ){

      return true;
    }
    return false;
  }

  public Boolean isTakenByPolice(int x, int y){
    for(Police t : this.polices){
      if( t.getX() == x && t.getY() == y ){

        return true;
      }
    }
    return false;
  }

  public Boolean isFree(int x, int y){
    if( !isTakenByThief(x, y) && !isTakenByPolice(x, y) ){

      return true;
    }else{
      return false;
    }
  }

  public Boolean isThiefInRange(int x, int y){
    if( Math.abs(this.thief.getX() - x) <= 2 && Math.abs(this.thief.getY() - y) <= 2 ){

      return true;
    }
    return false;
  }

  public Boolean timeMove(){
    Random rand = new Random();
    int prevX = this.thief.getX();
    int prevY = this.thief.getY();
    // Move Thief
    while(true){
      int xShift = rand.nextInt(3) - 1;
      while(true){
        if( this.thief.getX() + xShift >= 0 && this.thief.getX() + xShift < this.n ){

          break;
        }
        xShift = rand.nextInt(3) - 1;
      }
      int yShift = rand.nextInt(3) - 1;
      while(true){
        if( this.thief.getY() + yShift >= 0 && this.thief.getY() + yShift < this.m ){

          break;
        }
        yShift = rand.nextInt(3) - 1;
      }
      if( xShift != 0 || yShift != 0 ){

        this.thiefMoves++;
        this.thief.setCoordinates(this.thief.getX() + xShift, this.thief.getY() + yShift);
        if( isTakenByPolice(this.thief.getX() + xShift, this.thief.getY() + yShift) ){

          return false;
        }else{
          break;
        }
      }
    }
    // Move Polices
    for(Police p : this.polices){
      if( p.getTarget() == null){// Random Move

        while(true){
          int xShift = rand.nextInt(3) - 1;
          while(true){
            if( p.getX() + xShift >= 0 && p.getX() + xShift < this.n ){

              break;
            }
            xShift = rand.nextInt(3) - 1;
          }
          int yShift = rand.nextInt(3) - 1;
          while(true){
            if( p.getY() + yShift >= 0 && p.getY() + yShift < this.m ){

              break;
            }
            yShift = rand.nextInt(3) - 1;
          }
          if( xShift != 0 || yShift != 0 ){
            if( !isTakenByPolice(p.getX() + xShift, p.getY() + yShift) ){

              p.setCoordinates(p.getX() + xShift, p.getY() + yShift);
              if( isThiefInRange(p.getX(), p.getY()) ){

                for( Police police : this.polices ){
                  police.setTarget(this.thief);
                }
              }
              this.policeMoves++;
              break;
            }
          }
        }
      }else{// Move based on thought :D
        int xShiftThought = 0;
        int yShiftThought = 0;
        if( prevX - p.getX() != 0 ){

          xShiftThought = (prevX - p.getX()) / Math.abs(prevX - p.getX());
        }
        if( prevY - p.getY() != 0 ){

          yShiftThought = (prevY - p.getY()) / Math.abs(prevY - p.getY());
        }
        if( !isTakenByPolice(p.getX() + xShiftThought, p.getY() + yShiftThought) ){ // Diagonal

          p.setCoordinates(p.getX() + xShiftThought, p.getY() + yShiftThought);
          this.policeMoves++;
          if( p.getY() == prevY && p.getX() == prevX ){

            return false;
          }
        }else if( !isTakenByPolice(p.getX(), p.getY() + yShiftThought) ){ // Y Axis

          p.setCoordinates(p.getX(), p.getY() + yShiftThought);
          this.policeMoves++;
          if( p.getY() == prevY && p.getX() == prevX ){

            return false;
          }
        }else if( !isTakenByPolice(p.getX() + xShiftThought, p.getY()) ){ // X Axis

          p.setCoordinates(p.getX() + xShiftThought, p.getY());
          this.policeMoves++;
          if( p.getY() == prevY && p.getX() == prevX ){

            return false;
          }
        }else{ // Random Move
          while(true){
            int xShift = rand.nextInt(3) - 1;
            while(true){
              if( p.getX() + xShift >= 0 && p.getX() + xShift < this.n ){

                break;
              }
              xShift = rand.nextInt(3) - 1;
            }
            int yShift = rand.nextInt(3) - 1;
            while(true){
              if( p.getY() + yShift >= 0 && p.getY() + yShift < this.m ){

                break;
              }
              yShift = rand.nextInt(3) - 1;
            }
            if( xShift != 0 || yShift != 0 ){
              if( !isTakenByPolice(p.getX() + xShift, p.getY() + yShift) ){

                p.setCoordinates(p.getX() + xShift, p.getY() + yShift);
                this.policeMoves++;
                if( p.getY() == prevY && p.getX() == prevX ){

                  return false;
                }
                break;
              }
            }
          }
        }
      }
    }
    return true;
  }

  public void print(int t){
    System.out.println("Time: " + t);
    for(int i = 0; i < n; i++){
      for(int j = 0; j < m; j++){
        if( isTakenByThief(i, j) ){

          System.out.print("T  ");
        }else if( isTakenByPolice(i, j) ){
          System.out.print("P  ");
        }else{
          System.out.print("-  ");
        }
      }
      System.out.println("");
    }
  }

  public void printFinal(){
    System.out.println("\tThe Thief is caught!");
    System.out.println();
    System.out.println("Thief Moves: " + this.thiefMoves);
    System.out.println("Police Moves: " + this.policeMoves);
  }
}
